@@ -191,9 +191,8 @@ class OIDCAuthentication:
         logger.debug('received authentication response: %s', authn_resp.to_json())
 
         try:
-            result = AuthResponseHandler(client).process_auth_response(authn_resp,
-                                                                       flask.session.pop('state'),
-                                                                       flask.session.pop('nonce'))
+            auth_handler = AuthResponseHandler(client)
+            result = auth_handler.process_auth_response(authn_resp, flask.session.pop('state'), flask.session.pop('nonce'))
         except AuthResponseErrorResponseError as e:
             return self._handle_error_response(e.error_response, is_processing_fragment_encoded_response)
         except AuthResponseProcessError as e:
@@ -232,7 +231,7 @@ class OIDCAuthentication:
 
         return 'Something went wrong with the authentication, please try to login again.'
 
-    def oidc_auth(self, provider_name : str = None):
+    def oidc_auth(self, provider_name : str = None, bearer = False):
         """Authentication decorater. 
         
         Wraps a given Flask endpoint with OIDC authentication for the given provider.
@@ -240,31 +239,50 @@ class OIDCAuthentication:
         Args:
             provider_name str:
                 Name of the provider. If None, goes to default authentication.
+            bearer bool:
+                Whether access using a bearer token is allowed. Default is False.
 
         Raises:
             ValueError: 
                 If invalid provider name is provided.
         """        
-        if provider_name is None:
-            provider_name = self.default_provider
-        elif provider_name not in self._provider_configurations:
-            raise ValueError(
-                "Provider name '{}' not in configured providers: {}.".format(provider_name,
-                self._provider_configurations.keys())
-            )
 
         def oidc_decorator(view_func):
-            if provider_name is None:
+                PROVIDER_DECORATOR = provider_name
+                BEARER = bearer
                 @functools.wraps(view_func)
                 def wrapper(*args, **kwargs):
-                    return view_func(*args, **kwargs)
-                return wrapper
-            else:
-                @functools.wraps(view_func)
-                def wrapper(*args, **kwargs):
-                    session = UserSession(flask.session, provider_name)
+                    PROVIDER_NAME = PROVIDER_DECORATOR
+                    BEARER_ALLOWED = BEARER
+
+                    if PROVIDER_NAME is None:
+                        PROVIDER_NAME = self.default_provider
+                    elif PROVIDER_NAME not in self._provider_configurations:
+                        raise ValueError(
+                            "Provider name '{}' not in configured providers: {}.".format(provider_name,
+                            self._provider_configurations.keys())
+                        )
+
+                    if PROVIDER_NAME is None:
+                        return view_func(*args, **kwargs)
+                    
+                    session = UserSession(flask.session, PROVIDER_NAME)
                     client = self.clients[session.current_provider]
 
+                    if BEARER_ALLOWED and 'Authorization' in flask.request.headers.keys():
+                        auth_msg = str(flask.request.headers['Authorization'])
+
+                        if auth_msg.startswith('Bearer '):
+                            logger.debug('accessing through bearer token')
+                            token = auth_msg[7:]
+                            userinfo = client.userinfo_request(token)
+                            if 'error' in userinfo:
+                                return flask.Response('Invalid Bearer Token',  401, {'WWW-Authenticate':'Basic realm="Login Required"'})
+                            
+                            session.update(access_token=token, userinfo=userinfo.to_dict())
+                            return view_func(*args, **kwargs)
+
+
                     if session.should_refresh(client.session_refresh_interval_seconds):
                         logger.debug('user auth will be refreshed "silently"')
                         return self._authenticate(client, interactive=False)
@@ -274,9 +292,7 @@ class OIDCAuthentication:
                     else:
                         logger.debug('user not authenticated, start flow')
                         return self._authenticate(client)
-
                 return wrapper
-
         return oidc_decorator
 
     def _logout(self):
@@ -339,19 +355,8 @@ def dict_to_providers_config(provider_name : str, config_dict : dict ) -> Provid
     """    
     client_name = config_dict.get(provider_name + '_ISSUER')
     
-
-
     client_id = config_dict.get(provider_name + '_CLIENT', None)
     client_secret = config_dict.get(provider_name + '_SECRET', None)
-
-    if client_id is None and client_secret is None:
-        client_id = config_dict.get(provider_name.upper() + '_CLIENT', None)
-        client_secret = config_dict.get(provider_name.upper() + '_SECRET', None)
-
-    if client_id is None:
-        raise ValueError('No client id found for provider {}'.format(provider_name))
-
-
     client_metadata=ClientMetadata(client_id = client_id, client_secret=client_secret)
 
     return ProviderConfiguration(issuer=client_name,
